'use strict'

const sequencify = require('sequencify')
const readCsv = require('gtfs-utils/read-csv')
const {PassThrough, pipeline} = require('stream')
const formatters = require('./lib')
const deps = require('./lib/deps')
const pkg = require('./package.json')
const converter = require('./lib/converter')

const convertGtfsToSql = async (files, opt = {}) => {
	const {
		silent,
		requireDependencies,
		ignoreUnsupportedFiles,
	} = {
		silent: false,
		requireDependencies: false,
		ignoreUnsupportedFiles: false,
		...opt,
	}

	if (ignoreUnsupportedFiles) {
		files = files.filter(f => !!formatters[f.name])
	}
	const isAvailable = name => files.some(f => f.name === name)
	for (const {name} of files) {
		if (!formatters[name]) {
			throw new Error('invalid/unsupported file: ' + name)
		}
		if (requireDependencies) {
			for (const dep of deps[name] || []) {
				if (!isAvailable(dep)) {
					throw new Error(name + ' depends on ' + dep)
				}
			}
		}
	}

	const tasks = {}
	for (const file of files) {
		tasks[file.name] = {
			...file,
			dep: (deps[file.name] || []).filter(isAvailable)
		}
	}
	const order = []
	sequencify(tasks, files.map(f => f.name), order)

	process.stdout.write(`\
-- GTFS SQL dump generated by ${pkg.name} v${pkg.version}
-- ${pkg.homepage}
\\set ON_ERROR_STOP on;
CREATE EXTENSION IF NOT EXISTS postgis;
BEGIN;
\n`)

	for (const name of order) {
		if (!silent) console.error(name)
		process.stdout.write(`-- ${name}\n-----------------\n\n`)

		const {file} = tasks[name]
		const src = readCsv(file)
		const convert = converter(formatters[name])

		// This is ugly, but I'm a bit overwhelmed with the number of
		// edges cases in the stream/stdio logic.
		// https://gist.github.com/derhuerst/42ab81b6d8ea5e05b4e2bfe83f702216
		const dest = new PassThrough()
		dest.pipe(process.stdout)

		await new Promise((resolve, reject) => {
			const destroyDest = err => dest.destroy(err)
			process.stdout.on('error', destroyDest)
			pipeline(
				src,
				convert,
				dest,
				(err) => {
					process.stdout.removeListener('error', destroyDest)
					if (err) reject(err)
					else setTimeout(resolve, 100)
				},
			)
		})
	}

	process.stdout.write(`\
COMMIT;`)
}

module.exports = convertGtfsToSql
